using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using Syncfusion.Drawing;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;

using ISummary = Syncfusion.Collections.BinaryTree.ITreeTableSummary;

namespace CustomSectionInGroup
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl gridGroupingControl1;
		private CustomSectionInGroup.DataSet1 northwindDataSet1;
		private System.Data.SqlClient.SqlCommand sqlSelectCommand2;
		private System.Data.SqlClient.SqlCommand sqlInsertCommand2;
		private System.Data.SqlClient.SqlCommand sqlUpdateCommand2;
		private System.Data.SqlClient.SqlCommand sqlDeleteCommand2;
		private System.Data.SqlClient.SqlDataAdapter sqlDataAdapterOrders;
		private System.Data.SqlClient.SqlConnection sqlConnection2;

		private System.ComponentModel.Container components = null;

		public Form1()
		{
			// GroupingEngineFactory provides a modified GridChildTable that adds an extra section
			GridEngineFactory.Factory = new GroupingEngineFactory();
			
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//

			InitializeDataSet();


			GridTableDescriptor ordersDescriptor = this.gridGroupingControl1.TableDescriptor;


			// You can define a summary row and mark it hidden.
			// In that summary row you can add a column and set it's mapping name (and DisplayColumn) to be Freight

			GridSummaryColumnDescriptor summaryColumn1 = new GridSummaryColumnDescriptor("FreightAverage", SummaryType.DoubleAggregate, "Freight", "{Average:###.00}");
			GridSummaryRowDescriptor summaryRow1 = new GridSummaryRowDescriptor();
			summaryRow1.Name = "SummaryRow 1";
			summaryRow1.Visible = false;
			summaryRow1.SummaryColumns.Add(summaryColumn1);
			ordersDescriptor.SummaryRows.Add(summaryRow1);

			// This is a second row, not marked hidden and therefore shown at the end of the group.
			GridSummaryColumnDescriptor summaryColumn2 = new GridSummaryColumnDescriptor("FreightTotal", SummaryType.DoubleAggregate, "Freight", "{Sum:###.00}");
			GridSummaryRowDescriptor summaryRow2 = new GridSummaryRowDescriptor();
			summaryRow2.Name = "SummaryRow 2";
			summaryRow2.Title = "Total";
			summaryRow2.Visible = true;
			summaryRow2.SummaryColumns.Add(summaryColumn2);
			ordersDescriptor.SummaryRows.Add(summaryRow2);

			// Have Extra Section bar cover some cells.
			this.gridGroupingControl1.TableModel.QueryCoveredRange += new GridQueryCoveredRangeEventHandler(TableModel_QueryCoveredRange);

			// Provide contents to be displayed in cell
			this.gridGroupingControl1.QueryCellStyleInfo += new GridTableCellStyleInfoEventHandler(gridGroupingControl1_QueryCellStyleInfo);
			this.gridGroupingControl1.TableControlCellDrawn += new GridTableControlDrawCellEventHandler(gridGroupingControl1_TableControlCellDrawn);

			// Move Freight column a bit to the left 
			this.gridGroupingControl1.TableDescriptor.VisibleColumns.Move(this.gridGroupingControl1.TableDescriptor.VisibleColumns.IndexOf("Freight"), 4);


			this.gridGroupingControl1.TableDescriptor.GroupedColumns.Add("EmployeeID");
			this.gridGroupingControl1.TableDescriptor.GroupedColumns.Add("ShipCountry");

			this.gridGroupingControl1.TableControlCurrentCellActivated += new GridTableControlEventHandler(gridGroupingControl1_TableControlCurrentCellActivated);

			this.gridGroupingControl1.TableControlCellDoubleClick += new GridTableControlCellClickEventHandler(groupingGrid1_TableControlCellDoubleClick);
			this.gridGroupingControl1.TableOptions.AllowSortColumns = false;
			this.gridGroupingControl1.TableControlCellClick += new GridTableControlCellClickEventHandler(gridGroupingControl1_TableControlCellClick);
			this.gridGroupingControl1.TableControlPrepareViewStyleInfo += new GridTableControlPrepareViewStyleInfoEventHandler(gridGroupingControl1_TableControlPrepareViewStyleInfo);

			this.gridGroupingControl1.Table.ExpandAllGroups();

			foreach (Element el in this.gridGroupingControl1.Table.DisplayElements)
			{
				if (el is ExtraSection)
				{
					el.ParentTable.CurrentRecordManager.NavigateTo(el);
					break;
				}
			}
		}

		string xmlName = "Orders.xml";
		bool msdeAvailable = false;

		void InitializeDataSet()
		{
			if (msdeAvailable)
			{
				// Initialize the data set

				this.northwindDataSet1.BeginInit();

				// Order is important here - first fill tables that are referenced with 
				// foreign keys, then tables that have references to others.

				this.sqlDataAdapterOrders.Fill(this.northwindDataSet1);

				this.northwindDataSet1.EndInit();

				this.northwindDataSet1.WriteXml(xmlName);
			}
			else
			{
				// Read from a xml file instead. 
				if (!ReadXml(this.northwindDataSet1, xmlName))
				{
					// If no xml found, try again with MSDE
					msdeAvailable = true;
					InitializeDataSet();
				}
			}

		}

		bool ReadXml(DataSet ds, string xmlFileName)
		{
			// Check both in parent folder and Parent\Data folders.
			string xmlDataFileName = @"Data\" + xmlFileName;
			for (int n = 0; n < 10; n++)
			{
				if (System.IO.File.Exists(xmlFileName))
				{
					ds.ReadXml(xmlFileName);
					return true;
				}
				if (System.IO.File.Exists(xmlDataFileName))
				{
					ds.ReadXml(xmlDataFileName);
					return true;
				}
				xmlFileName = @"..\" + xmlFileName;
				xmlDataFileName = @"..\" + xmlDataFileName;
			}

			return false;
		}


		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.northwindDataSet1 = new CustomSectionInGroup.DataSet1();
			this.gridGroupingControl1 = new Syncfusion.Windows.Forms.Grid.Grouping.GridGroupingControl();
			this.sqlSelectCommand2 = new System.Data.SqlClient.SqlCommand();
			this.sqlInsertCommand2 = new System.Data.SqlClient.SqlCommand();
			this.sqlUpdateCommand2 = new System.Data.SqlClient.SqlCommand();
			this.sqlDeleteCommand2 = new System.Data.SqlClient.SqlCommand();
			this.sqlDataAdapterOrders = new System.Data.SqlClient.SqlDataAdapter();
			this.sqlConnection2 = new System.Data.SqlClient.SqlConnection();
			((System.ComponentModel.ISupportInitialize)(this.northwindDataSet1)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.gridGroupingControl1)).BeginInit();
			this.SuspendLayout();
			// 
			// northwindDataSet1
			// 
			this.northwindDataSet1.DataSetName = "DataSet1";
			this.northwindDataSet1.Locale = new System.Globalization.CultureInfo("en-US");
			// 
			// gridGroupingControl1
			// 
			this.gridGroupingControl1.DataSource = this.northwindDataSet1.Orders;
			this.gridGroupingControl1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.gridGroupingControl1.Location = new System.Drawing.Point(0, 0);
			this.gridGroupingControl1.Name = "gridGroupingControl1";
			this.gridGroupingControl1.ShowGroupDropArea = true;
			this.gridGroupingControl1.Size = new System.Drawing.Size(690, 664);
			this.gridGroupingControl1.TabIndex = 0;
			// 
			// sqlSelectCommand2
			// 
			this.sqlSelectCommand2.CommandText = "SELECT OrderID, CustomerID, EmployeeID, OrderDate, RequiredDate, ShippedDate, Shi" +
				"pVia, Freight, ShipName, ShipAddress, ShipCity, ShipRegion, ShipPostalCode, Ship" +
				"Country FROM Orders";
			this.sqlSelectCommand2.Connection = this.sqlConnection2;
			// 
			// sqlInsertCommand2
			// 
			this.sqlInsertCommand2.CommandText = @"INSERT INTO Orders(CustomerID, EmployeeID, OrderDate, RequiredDate, ShippedDate, ShipVia, Freight, ShipName, ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry) VALUES (@CustomerID, @EmployeeID, @OrderDate, @RequiredDate, @ShippedDate, @ShipVia, @Freight, @ShipName, @ShipAddress, @ShipCity, @ShipRegion, @ShipPostalCode, @ShipCountry); SELECT OrderID, CustomerID, EmployeeID, OrderDate, RequiredDate, ShippedDate, ShipVia, Freight, ShipName, ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry FROM Orders WHERE (OrderID = @@IDENTITY)";
			this.sqlInsertCommand2.Connection = this.sqlConnection2;
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@CustomerID", System.Data.SqlDbType.NVarChar, 5, "CustomerID"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@EmployeeID", System.Data.SqlDbType.Int, 4, "EmployeeID"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@OrderDate", System.Data.SqlDbType.DateTime, 8, "OrderDate"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@RequiredDate", System.Data.SqlDbType.DateTime, 8, "RequiredDate"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShippedDate", System.Data.SqlDbType.DateTime, 8, "ShippedDate"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipVia", System.Data.SqlDbType.Int, 4, "ShipVia"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Freight", System.Data.SqlDbType.Money, 8, "Freight"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipName", System.Data.SqlDbType.NVarChar, 40, "ShipName"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipAddress", System.Data.SqlDbType.NVarChar, 60, "ShipAddress"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipCity", System.Data.SqlDbType.NVarChar, 15, "ShipCity"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipRegion", System.Data.SqlDbType.NVarChar, 15, "ShipRegion"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipPostalCode", System.Data.SqlDbType.NVarChar, 10, "ShipPostalCode"));
			this.sqlInsertCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipCountry", System.Data.SqlDbType.NVarChar, 15, "ShipCountry"));
			// 
			// sqlUpdateCommand2
			// 
			this.sqlUpdateCommand2.CommandText = "UPDATE Orders SET CustomerID = @CustomerID, EmployeeID = @EmployeeID, OrderDate =" +
				" @OrderDate, RequiredDate = @RequiredDate, ShippedDate = @ShippedDate, ShipVia =" +
				" @ShipVia, Freight = @Freight, ShipName = @ShipName, ShipAddress = @ShipAddress," +
				" ShipCity = @ShipCity, ShipRegion = @ShipRegion, ShipPostalCode = @ShipPostalCod" +
				"e, ShipCountry = @ShipCountry WHERE (OrderID = @Original_OrderID) AND (CustomerI" +
				"D = @Original_CustomerID OR @Original_CustomerID IS NULL AND CustomerID IS NULL)" +
				" AND (EmployeeID = @Original_EmployeeID OR @Original_EmployeeID IS NULL AND Empl" +
				"oyeeID IS NULL) AND (Freight = @Original_Freight OR @Original_Freight IS NULL AN" +
				"D Freight IS NULL) AND (OrderDate = @Original_OrderDate OR @Original_OrderDate I" +
				"S NULL AND OrderDate IS NULL) AND (RequiredDate = @Original_RequiredDate OR @Ori" +
				"ginal_RequiredDate IS NULL AND RequiredDate IS NULL) AND (ShipAddress = @Origina" +
				"l_ShipAddress OR @Original_ShipAddress IS NULL AND ShipAddress IS NULL) AND (Shi" +
				"pCity = @Original_ShipCity OR @Original_ShipCity IS NULL AND ShipCity IS NULL) A" +
				"ND (ShipCountry = @Original_ShipCountry OR @Original_ShipCountry IS NULL AND Shi" +
				"pCountry IS NULL) AND (ShipName = @Original_ShipName OR @Original_ShipName IS NU" +
				"LL AND ShipName IS NULL) AND (ShipPostalCode = @Original_ShipPostalCode OR @Orig" +
				"inal_ShipPostalCode IS NULL AND ShipPostalCode IS NULL) AND (ShipRegion = @Origi" +
				"nal_ShipRegion OR @Original_ShipRegion IS NULL AND ShipRegion IS NULL) AND (Ship" +
				"Via = @Original_ShipVia OR @Original_ShipVia IS NULL AND ShipVia IS NULL) AND (S" +
				"hippedDate = @Original_ShippedDate OR @Original_ShippedDate IS NULL AND ShippedD" +
				"ate IS NULL); SELECT OrderID, CustomerID, EmployeeID, OrderDate, RequiredDate, S" +
				"hippedDate, ShipVia, Freight, ShipName, ShipAddress, ShipCity, ShipRegion, ShipP" +
				"ostalCode, ShipCountry FROM Orders WHERE (OrderID = @OrderID)";
			this.sqlUpdateCommand2.Connection = this.sqlConnection2;
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@CustomerID", System.Data.SqlDbType.NVarChar, 5, "CustomerID"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@EmployeeID", System.Data.SqlDbType.Int, 4, "EmployeeID"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@OrderDate", System.Data.SqlDbType.DateTime, 8, "OrderDate"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@RequiredDate", System.Data.SqlDbType.DateTime, 8, "RequiredDate"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShippedDate", System.Data.SqlDbType.DateTime, 8, "ShippedDate"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipVia", System.Data.SqlDbType.Int, 4, "ShipVia"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Freight", System.Data.SqlDbType.Money, 8, "Freight"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipName", System.Data.SqlDbType.NVarChar, 40, "ShipName"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipAddress", System.Data.SqlDbType.NVarChar, 60, "ShipAddress"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipCity", System.Data.SqlDbType.NVarChar, 15, "ShipCity"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipRegion", System.Data.SqlDbType.NVarChar, 15, "ShipRegion"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipPostalCode", System.Data.SqlDbType.NVarChar, 10, "ShipPostalCode"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@ShipCountry", System.Data.SqlDbType.NVarChar, 15, "ShipCountry"));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_OrderID", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "OrderID", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_CustomerID", System.Data.SqlDbType.NVarChar, 5, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "CustomerID", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_EmployeeID", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "EmployeeID", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_Freight", System.Data.SqlDbType.Money, 8, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "Freight", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_OrderDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "OrderDate", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_RequiredDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "RequiredDate", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipAddress", System.Data.SqlDbType.NVarChar, 60, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipAddress", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipCity", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipCity", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipCountry", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipCountry", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipName", System.Data.SqlDbType.NVarChar, 40, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipName", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipPostalCode", System.Data.SqlDbType.NVarChar, 10, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipPostalCode", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipRegion", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipRegion", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipVia", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipVia", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShippedDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShippedDate", System.Data.DataRowVersion.Original, null));
			this.sqlUpdateCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@OrderID", System.Data.SqlDbType.Int, 4, "OrderID"));
			// 
			// sqlDeleteCommand2
			// 
			this.sqlDeleteCommand2.CommandText = @"DELETE FROM Orders WHERE (OrderID = @Original_OrderID) AND (CustomerID = @Original_CustomerID OR @Original_CustomerID IS NULL AND CustomerID IS NULL) AND (EmployeeID = @Original_EmployeeID OR @Original_EmployeeID IS NULL AND EmployeeID IS NULL) AND (Freight = @Original_Freight OR @Original_Freight IS NULL AND Freight IS NULL) AND (OrderDate = @Original_OrderDate OR @Original_OrderDate IS NULL AND OrderDate IS NULL) AND (RequiredDate = @Original_RequiredDate OR @Original_RequiredDate IS NULL AND RequiredDate IS NULL) AND (ShipAddress = @Original_ShipAddress OR @Original_ShipAddress IS NULL AND ShipAddress IS NULL) AND (ShipCity = @Original_ShipCity OR @Original_ShipCity IS NULL AND ShipCity IS NULL) AND (ShipCountry = @Original_ShipCountry OR @Original_ShipCountry IS NULL AND ShipCountry IS NULL) AND (ShipName = @Original_ShipName OR @Original_ShipName IS NULL AND ShipName IS NULL) AND (ShipPostalCode = @Original_ShipPostalCode OR @Original_ShipPostalCode IS NULL AND ShipPostalCode IS NULL) AND (ShipRegion = @Original_ShipRegion OR @Original_ShipRegion IS NULL AND ShipRegion IS NULL) AND (ShipVia = @Original_ShipVia OR @Original_ShipVia IS NULL AND ShipVia IS NULL) AND (ShippedDate = @Original_ShippedDate OR @Original_ShippedDate IS NULL AND ShippedDate IS NULL)";
			this.sqlDeleteCommand2.Connection = this.sqlConnection2;
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_OrderID", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "OrderID", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_CustomerID", System.Data.SqlDbType.NVarChar, 5, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "CustomerID", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_EmployeeID", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "EmployeeID", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_Freight", System.Data.SqlDbType.Money, 8, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "Freight", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_OrderDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "OrderDate", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_RequiredDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "RequiredDate", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipAddress", System.Data.SqlDbType.NVarChar, 60, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipAddress", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipCity", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipCity", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipCountry", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipCountry", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipName", System.Data.SqlDbType.NVarChar, 40, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipName", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipPostalCode", System.Data.SqlDbType.NVarChar, 10, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipPostalCode", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipRegion", System.Data.SqlDbType.NVarChar, 15, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipRegion", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShipVia", System.Data.SqlDbType.Int, 4, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShipVia", System.Data.DataRowVersion.Original, null));
			this.sqlDeleteCommand2.Parameters.Add(new System.Data.SqlClient.SqlParameter("@Original_ShippedDate", System.Data.SqlDbType.DateTime, 8, System.Data.ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "ShippedDate", System.Data.DataRowVersion.Original, null));
			// 
			// sqlDataAdapterOrders
			// 
			this.sqlDataAdapterOrders.DeleteCommand = this.sqlDeleteCommand2;
			this.sqlDataAdapterOrders.InsertCommand = this.sqlInsertCommand2;
			this.sqlDataAdapterOrders.SelectCommand = this.sqlSelectCommand2;
			this.sqlDataAdapterOrders.TableMappings.AddRange(new System.Data.Common.DataTableMapping[] {
																										   new System.Data.Common.DataTableMapping("Table", "Orders", new System.Data.Common.DataColumnMapping[] {
																																																					 new System.Data.Common.DataColumnMapping("OrderID", "OrderID"),
																																																					 new System.Data.Common.DataColumnMapping("CustomerID", "CustomerID"),
																																																					 new System.Data.Common.DataColumnMapping("EmployeeID", "EmployeeID"),
																																																					 new System.Data.Common.DataColumnMapping("OrderDate", "OrderDate"),
																																																					 new System.Data.Common.DataColumnMapping("RequiredDate", "RequiredDate"),
																																																					 new System.Data.Common.DataColumnMapping("ShippedDate", "ShippedDate"),
																																																					 new System.Data.Common.DataColumnMapping("ShipVia", "ShipVia"),
																																																					 new System.Data.Common.DataColumnMapping("Freight", "Freight"),
																																																					 new System.Data.Common.DataColumnMapping("ShipName", "ShipName"),
																																																					 new System.Data.Common.DataColumnMapping("ShipAddress", "ShipAddress"),
																																																					 new System.Data.Common.DataColumnMapping("ShipCity", "ShipCity"),
																																																					 new System.Data.Common.DataColumnMapping("ShipRegion", "ShipRegion"),
																																																					 new System.Data.Common.DataColumnMapping("ShipPostalCode", "ShipPostalCode"),
																																																					 new System.Data.Common.DataColumnMapping("ShipCountry", "ShipCountry")})});
			this.sqlDataAdapterOrders.UpdateCommand = this.sqlUpdateCommand2;
			// 
			// sqlConnection2
			// 
			this.sqlConnection2.ConnectionString = "data source=(local)\\NETSDK;initial catalog=Northwind;integrated security=SSPI;per" +
				"sist security info=True;workstation id=localhost;packet size=4096";
			// 
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(690, 664);
			this.Controls.Add(this.gridGroupingControl1);
			this.Name = "Form1";
			this.Text = "Syncfusion Essential Grid - CustomSectionInGroup";
			((System.ComponentModel.ISupportInitialize)(this.northwindDataSet1)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.gridGroupingControl1)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		int extraSectionCoverCols = 3;

		private void TableModel_QueryCoveredRange(object sender, GridQueryCoveredRangeEventArgs e)
		{
			GridTable thisTable = this.gridGroupingControl1.Table;
			if (e.RowIndex < thisTable.DisplayElements.Count)
			{
				Element el = thisTable.DisplayElements[e.RowIndex];

				if (el is ExtraSection)
				{
					// Cover some cells of the extra section (specified with extraSectionCoverCols)
					int startCol = el.GroupLevel + 1;  // Add +1 so we have place for column header
					if (e.ColIndex >= startCol && e.ColIndex <= this.extraSectionCoverCols + el.ParentTableDescriptor.GroupedColumns.Count)
					{
						e.Range = GridRangeInfo.Cells(e.RowIndex, startCol, e.RowIndex, this.extraSectionCoverCols + el.ParentTableDescriptor.GroupedColumns.Count);
						e.Handled = true;
					}
				}
			}
		}

		private void gridGroupingControl1_QueryCellStyleInfo(object sender, GridTableCellStyleInfoEventArgs e)
		{
            GridGroupingControl groupingControl = (GridGroupingControl) sender;
			Element el = e.TableCellIdentity.DisplayElement;

			e.Style.CellTipText = el.GetType().Name + (el.ParentElement != null ? " (Parent: " + el.ParentElement.GetType().Name + ")" : "");

			if (el is ExtraSection)
			{
				if (e.Style.TableCellIdentity.ColIndex == 0)
				{
					// Row Header
					e.Style.CellType = ((GridTableDescriptor) el.ParentTableDescriptor).Appearance.RecordRowHeaderCell.CellType;

					if (el == el.ParentTable.CurrentElement)
						e.Style.Text = "#";
				}
				else if (e.Style.TableCellIdentity.ColIndex <= el.GroupLevel)
				{
					// Group Indent 
					e.Style.BackColor = Color.FromArgb( 218, 229, 245 );
				}
				else if (e.Style.TableCellIdentity.ColIndex <= extraSectionCoverCols + el.ParentTableDescriptor.GroupedColumns.Count)
				{
					// Covered area (see TableModel_QueryCoveredRange above)
					e.Style.BackColor = Color.FromArgb( 192, 201, 219 );
					e.Style.CellValue = "ExtraSection (" + el.ParentGroup.Name + ")";
				}
				else if (e.Style.TableCellIdentity.ColIndex > 3)
				{
					e.Style.BackColor = Color.FromArgb( 252, 172, 38 );
					// you can get the column as follows:
                    GridTable table = (GridTable) el.ParentTable;
                    GridTableDescriptor td = e.TableCellIdentity.Table.TableDescriptor;
                    GridColumnDescriptor column = td.RecordRowColumns[0, e.Style.TableCellIdentity.ColIndex - el.ParentTableDescriptor.GroupedColumns.Count - 1];
                    e.Style.CellValue = column.Name;

                    // Using that column you could try and identify the summary that should be displayed in this cell.
                    if( column.MappingName == "Freight")
                    {
                        // Calling this method to demonstrate different alternatives to get to the summary text
                        e.Style.Text = GetSummaryText(el.ParentGroup, "SummaryRow 1", "FreightAverage");

                        // Easier is to simple call built-in routine:
                        // e.Style.Text = GridEngine.GetSummaryText(el.ParentGroup, "SummaryRow 1", "FreightAverage");
                    }	
                }
			}
		}

        /// <summary>
        /// Demonstrates different alternatives to get to the summary text (strong typed vs GridSummaryColumnDescriptor.Format)
        /// </summary>
        public string GetSummaryText(Group group, string summaryRowName, string summaryColumnName)
        {
            GridTable table = (GridTable) group.ParentTable;
            GridTableDescriptor td = table.TableDescriptor;
            GridSummaryRowDescriptor srd = td.SummaryRows[summaryRowName];
            GridSummaryColumnDescriptor scd = srd.SummaryColumns[summaryColumnName];

            return GetSummaryText(group, scd);
        }

        /// <summary>
        /// Demonstrates different alternatives to get to the summary text (strong typed vs GridSummaryColumnDescriptor.Format)
        /// </summary>
        string GetSummaryText(Group group, GridSummaryColumnDescriptor scd)
        {
            GridTable table = (GridTable) group.ParentTable;
            GridTableDescriptor td = table.TableDescriptor;

            string summaryText = "";

            bool use31Code = true;
//            if (use31Code)
//            {
//                if (scd != null)
//                {
//                    // Option 1: GetDisplayText - this is actually the code used when you simply would call 
//                    // e.Style.Text = ((GridTable) table).GetSummaryText(group, "SummaryRow 1", "FreightAverage");
//                    //
//                    // Text is formatted as defined in GridSummaryColumnDescriptor.Format 
//                    summaryText = scd.GetDisplayText(group);  
//
//                    // or Option 2: Strong typed access to DoubleAggregateSummary.
//                    DoubleAggregateSummary summary1 = (DoubleAggregateSummary) group.GetSummary(scd.SummaryDescriptor);
//                    summaryText = string.Format("{0:c}", summary1.Average);
//
//                    // or Option 3: Use reflection to get "Average" property of summary
//                    summaryText = string.Format("{0:c}", group.GetSummaryProperty(scd.SummaryDescriptor, "Average"));
//                }
//
//            }
//            else
            {
                // This is the code you had to use in version 3.0 and earlier (still working but bit more complicate)
                if (scd != null)
                {
                    SummaryDescriptor sd1 = scd.SummaryDescriptor;
                    if (sd1 != null)
                    {
                        int indexOfSd1 = table.TableDescriptor.Summaries.IndexOf(sd1);

                        ISummary sum1 = group.GetSummaries(table)[indexOfSd1];
                        string text1 = scd.GetDisplayText(sum1);
                        summaryText = text1;

                        // - or - (access value directly)
                        // strong typed - you have to cast to Int32AggregateSummary.

                        DoubleAggregateSummary summary1 = (DoubleAggregateSummary) group.GetSummaries(table)[indexOfSd1];
                        summaryText = string.Format("{0:c}", summary1.Average);
                    }
                }
            }

            return summaryText;
        }

        /// <summary>
        /// This code works with base engine, no dependency on GridEngine / GridSummaryColumnDescriptor etc.
        /// </summary>
        string GetAverageSummary(SummaryDescriptor summaryDescriptor, Group group)
        {
            Table table = group.ParentTable;
            TableDescriptor td = table.TableDescriptor;
            string summaryText = "";

            bool use31Code = true;
//            if (use31Code)
//            {
//                // Option 1: Strong typed access to DoubleAggregateSummary.
//                DoubleAggregateSummary summary1 = (DoubleAggregateSummary) group.GetSummary(summaryDescriptor);
//                summaryText = string.Format("{0:c}", summary1.Average);
//
//                // or Option 2: Use reflection to get "Average" property of summary
//                summaryText = string.Format("{0:c}", group.GetSummaryProperty(summaryDescriptor, "Average"));
//            }
//
//            else
            {
                // This is the code you had to use in version 3.0 and earlier (still working but bit more complicate)
                if (summaryDescriptor != null)
                {
                    int indexOfSd1 = table.TableDescriptor.Summaries.IndexOf(summaryDescriptor);

                    // strong typed - you have to cast to DoubleAggregateSummary.

                    DoubleAggregateSummary summary1 = (DoubleAggregateSummary) group.GetSummaries(table)[indexOfSd1];
                    summaryText = string.Format("{0:c}", summary1.Average);
                }
            }
            return summaryText;
        }

		private void gridGroupingControl1_TableControlCurrentCellActivated(object sender, GridTableControlEventArgs e)
		{
			// Navigate to ExtraSection when textbox inside ExtraSection has become CurrentCell.

			GridCurrentCell gcc = e.TableControl.CurrentCell;
			GridTableCellStyleInfo style = e.TableControl.Model[gcc.RowIndex, gcc.ColIndex];
			GridTableCellStyleInfoIdentity id = style.TableCellIdentity;

			if (id.DisplayElement is ExtraSection)
				id.Table.CurrentRecordManager.NavigateTo(id.DisplayElement);
		}

		private void gridGroupingControl1_TableControlCellDrawn(object sender, GridTableControlDrawCellEventArgs e)
		{
			GridTableCellStyleInfo style = e.TableControl.Model[e.Inner.RowIndex, e.Inner.ColIndex];
			GridTableCellStyleInfoIdentity id = style.TableCellIdentity;

			Element el = id.DisplayElement;

			if (el is ExtraSection)
			{
				if (id.ColIndex == 0)
				{
					// Row Header and ExtraSection is CurrentElement
					if (el == el.ParentTable.CurrentElement)
					{
						// Draw Record indicator 
						try
						{
							Rectangle iconBounds = Rectangle.FromLTRB(e.Inner.Bounds.Right-15, e.Inner.Bounds.Top, e.Inner.Bounds.Right, e.Inner.Bounds.Bottom);
							iconBounds.Offset(-2, 0);
							IconPainter.PaintIcon(e.Inner.Graphics, iconBounds, Point.Empty, "SFARROW.BMP", style.TextColor);
						}
						catch
						{}
					}
				}
			}
		}

		[ThreadStaticAttribute] static IconPaint iconPainter;

		internal static IconPaint IconPainter
		{
			get
			{
				string ns = "CustomSectionInGroup";  // Must be same as your project settings Root Namespace
				if (iconPainter == null)
					iconPainter = new IconPaint(ns + ".", typeof(Form1).Assembly);
				return iconPainter;
			}
		}

		private void groupingGrid1_TableControlCellDoubleClick(object sender, GridTableControlCellClickEventArgs e)
		{
			Console.WriteLine("groupingGrid1_TableControlCellDoubleClick");
			Element el = e.TableControl.Table.NestedDisplayElements[e.Inner.RowIndex];
			if (el != null && el.Kind == DisplayElementKind.ColumnHeader)
			{
				highlightColumn = "";

				GridTableCellStyleInfo style = e.TableControl.Model[e.Inner.RowIndex, e.Inner.ColIndex];
				if (style.TableCellIdentity.Column != null)
				{
					GridTableDescriptor td = e.TableControl.TableDescriptor;
					string fieldName = style.TableCellIdentity.Column.MappingName;
					if (td.SortedColumns.Contains(fieldName))
					{
						switch (td.SortedColumns[fieldName].SortDirection)
						{
							case ListSortDirection.Ascending:
								td.SortedColumns[fieldName].SortDirection = ListSortDirection.Descending;
								break;
							case ListSortDirection.Descending:
								td.SortedColumns[fieldName].SortDirection = ListSortDirection.Ascending;
								break;
						}
					}
					else
					{
						e.TableControl.TableDescriptor.SortedColumns.Clear();
						e.TableControl.TableDescriptor.SortedColumns.Add(style.TableCellIdentity.Column.MappingName);
					}
				}
			}

		}

		string highlightColumn = "";

		private void gridGroupingControl1_TableControlCellClick(object sender, GridTableControlCellClickEventArgs e)
		{
			Console.WriteLine("gridGroupingControl1_TableControlCellClick");
			Element el = e.TableControl.Table.NestedDisplayElements[e.Inner.RowIndex];
			if (el != null && el.Kind == DisplayElementKind.ColumnHeader)
			{
				GridTableCellStyleInfo style = e.TableControl.Model[e.Inner.RowIndex, e.Inner.ColIndex];
				if (style.TableCellIdentity.Column != null)
				{
					if (highlightColumn != null && highlightColumn == style.TableCellIdentity.Column.Name)
						highlightColumn = "";
					else
						highlightColumn = style.TableCellIdentity.Column.Name;

					e.TableControl.Invalidate();
				}
			}
		}

		private void gridGroupingControl1_TableControlPrepareViewStyleInfo(object sender, GridTableControlPrepareViewStyleInfoEventArgs e)
		{
			GridTableCellStyleInfo style = (GridTableCellStyleInfo) e.Inner.Style;
			if (style.TableCellIdentity.Column != null)
			{
				if (highlightColumn != null && highlightColumn == style.TableCellIdentity.Column.Name)
				{
					style.BackColor = SystemColors.Highlight;
					style.TextColor = SystemColors.HighlightText;
				}
			}

		}
	}

	/// <summary>
	/// Mouse controller that provides support for dragging column headers within the
	/// GridTableControl or dragging them to the GroupDropArea.
	/// </summary>
	public class CustomTableControlDragHeaderMouseController : GridTableControlDragHeaderMouseController 
	{
		/// <summary>
		/// Initializes the mouse controller with grid it operates on
		/// </summary>
		/// <param name="grid"></param>
		public CustomTableControlDragHeaderMouseController(GridTableControl grid)
			: base(grid)
		{
		}

		public override void MouseDown(MouseEventArgs e)
		{
			Console.WriteLine("MouseDown " + e.Clicks.ToString());

			GridTableControl grid = this.GetGridTableControl();
			int rowIndex, colIndex;
			grid.PointToRowCol(new Point(e.X, e.Y), out rowIndex, out colIndex);

			if (e.Clicks == 2)
			{
				grid.RaiseCellDoubleClick(rowIndex, colIndex, e);
				return;
			}

			base.MouseDown (e);
		}

		public override void MouseMove(MouseEventArgs e)
		{
			Console.WriteLine("MouseMove");
			base.MouseMove (e);
		}

		public override void MouseUp(MouseEventArgs e)
		{
			Console.WriteLine("MouseUp " + e.Clicks.ToString());
			base.MouseUp (e);
		}

	}


	/// <summary>
	/// Derived GridTableControl (instantiade from CustomEngine.CreateTableControl override)
	/// wires CustomTableControlDragHeaderMouseController with grouping grid.  
	/// </summary>
	public class CustomTableControl : GridTableControl
	{
		public CustomTableControl(GridModel model)
			: base(model)
		{
		}

		bool inInitLayout = false;

		protected override void InitLayout()
		{
			try
			{
				inInitLayout = true;
				base.InitLayout();
			}
			finally
			{
				inInitLayout = false;
			}
		}

		public override void Initialize()
		{
			base.Initialize ();

			if (!inInitLayout)
			{
				this.MouseControllerDispatcher.Remove(this.MouseControllerDispatcher.Find("DragGroupHeader"));
				CustomTableControlDragHeaderMouseController mc = new CustomTableControlDragHeaderMouseController(this);
				this.MouseControllerDispatcher.Add(mc);
			}
		}

	}

	/// <summary>
	/// Derived GridNestedTableControl (instantiade from CustomEngine.CreateNestedTableControl override)
	/// wires CustomTableControlDragHeaderMouseController with grouping grid. 
	/// </summary>
	public class CustomNestedTableControl : GridNestedTableControl
	{
		public CustomNestedTableControl(GridTableModel relatedTableModel, GridTableControl parentGrid, GridNestedTableControlCellRenderer parentRenderer)
			: base(relatedTableModel, parentGrid, parentRenderer)
		{
		}

		bool inInitLayout = false;

		protected override void InitLayout()
		{
			try
			{
				inInitLayout = true;
				base.InitLayout();
			}
			finally
			{
				inInitLayout = false;
			}
		}

		public override void Initialize()
		{
			base.Initialize ();

			if (!inInitLayout)
			{
				this.MouseControllerDispatcher.Remove(this.MouseControllerDispatcher.Find("DragGroupHeader"));
				CustomTableControlDragHeaderMouseController mc = new CustomTableControlDragHeaderMouseController(this);
				this.MouseControllerDispatcher.Add(mc);
			}
		}

	}

}